# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .create_multi_response_bundle import CreateMultiResponseBundle
from .create_multi_response_resources_item import CreateMultiResponseResourcesItem


class CreateMultiResponse(UniversalBaseModel):
    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether extraction was successful
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message
    """

    bundle: typing.Optional[CreateMultiResponseBundle] = pydantic.Field(default=None)
    """
    FHIR transaction Bundle containing all extracted resources
    """

    resources: typing.Optional[typing.List[CreateMultiResponseResourcesItem]] = pydantic.Field(default=None)
    """
    Summary of extracted resources
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
