# This file was auto-generated by Fern from our API Definition.

import typing

FhirPatchRequestBodyItemOp = typing.Union[
    typing.Literal["add", "remove", "replace", "move", "copy", "test"], typing.Any
]
