# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .workflow_step_summary_type import WorkflowStepSummaryType


class WorkflowStepSummary(UniversalBaseModel):
    """
    Simplified workflow step without implementation details
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the step
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable name for the step
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detailed description of what the step does
    """

    type: typing.Optional[WorkflowStepSummaryType] = pydantic.Field(default=None)
    """
    Type of operation this step performs
    """

    provider_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Selected FHIR provider ID for this step
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
