# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .execute_workflow_response_results import ExecuteWorkflowResponseResults


class ExecuteWorkflowResponse(UniversalBaseModel):
    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the workflow execution was successful
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message with execution details
    """

    results: typing.Optional[ExecuteWorkflowResponseResults] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
