# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .workflow_definition import WorkflowDefinition
from .workflow_response import WorkflowResponse


class WorkflowsUpdateResponse(UniversalBaseModel):
    success: typing.Optional[bool] = None
    message: typing.Optional[str] = None
    workflow: typing.Optional[WorkflowResponse] = pydantic.Field(default=None)
    """
    Simplified workflow response with only essential step information
    """

    workflow_details: typing.Optional[WorkflowDefinition] = pydantic.Field(default=None)
    """
    Only included when verbose=true - contains full implementation details
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
