# This file was auto-generated by Fern from our API Definition.

import typing

Lang2FhirAndCreateRequestResource = typing.Union[
    typing.Literal[
        "auto",
        "appointment",
        "condition-encounter-diagnosis",
        "medicationrequest",
        "careplan",
        "condition-problems-health-concerns",
        "coverage",
        "encounter",
        "observation-clinical-result",
        "observation-lab",
        "patient",
        "procedure",
        "questionnaire",
        "questionnaireresponse",
        "simple-observation",
        "vital-signs",
    ],
    typing.Any,
]
