# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .code_response import CodeResponse
from .code_system_info import CodeSystemInfo


class ListCodesResponse(UniversalBaseModel):
    system: CodeSystemInfo
    codes: typing.List[CodeResponse]
    next_cursor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Cursor for fetching the next page (null if no more results)
    """

    has_more: bool = pydantic.Field()
    """
    Whether there are more results available
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
