# This file was auto-generated by Fern from our API Definition.

import typing

from .fhir_provider_sandbox_info import FhirProviderSandboxInfo
from .fhir_provider_template import FhirProviderTemplate

FhirProviderListResponseFhirProvidersItem = typing.Union[FhirProviderTemplate, FhirProviderSandboxInfo]
