# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .bad_request_error_body import BadRequestErrorBody
from .unauthorized_error_body import UnauthorizedErrorBody

__all__ = ["BadRequestErrorBody", "UnauthorizedErrorBody"]
