# This file was auto-generated by Fern from our API Definition.

import typing

AuthMethod = typing.Union[
    typing.Literal["client_secret", "google_healthcare", "jwt", "on_behalf_of", "token_passthrough", "none"], typing.Any
]
