# This file was auto-generated by Fern from our API Definition.

import typing

SearchResponseResourceType = typing.Union[
    typing.Literal[
        "AllergyIntolerance",
        "Appointment",
        "CarePlan",
        "CareTeam",
        "Condition",
        "Coverage",
        "Device",
        "DiagnosticReport",
        "DocumentReference",
        "Encounter",
        "Goal",
        "Immunization",
        "Location",
        "Medication",
        "MedicationRequest",
        "Observation",
        "Organization",
        "Patient",
        "PlanDefinition",
        "Practitioner",
        "PractitionerRole",
        "Procedure",
        "Provenance",
        "Questionnaire",
        "QuestionnaireResponse",
        "RelatedPerson",
        "Schedule",
        "ServiceRequest",
        "Slot",
        "Specimen",
    ],
    typing.Any,
]
