# This file was auto-generated by Fern from our API Definition.

import typing

DocumentRequestFileType = typing.Union[
    typing.Literal["application/pdf", "image/png", "image/jpeg", "image/jpg"], typing.Any
]
