# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .code_system_info import CodeSystemInfo
from .semantic_search_result import SemanticSearchResult


class SemanticSearchResponse(UniversalBaseModel):
    system: CodeSystemInfo
    results: typing.List[SemanticSearchResult] = pydantic.Field()
    """
    Codes ordered by semantic similarity (most similar first)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
