# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .code_system_info import CodeSystemInfo
from .text_search_result import TextSearchResult


class TextSearchResponse(UniversalBaseModel):
    system: CodeSystemInfo
    results: typing.List[TextSearchResult] = pydantic.Field()
    """
    Codes matching the search query
    """

    found: int = pydantic.Field()
    """
    Total number of matching results (may exceed results array due to pagination)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
